# Ruby Component-Entity-System

Ruby-CES, or CES for short, is a Ruby implementation for the Component-Entity-System architecture, commonly found in game engines but applicable to anything that might reuse a lot of the same code.
[This](https://www.raywenderlich.com/24878/introduction-to-component-based-architecture-in-games) article explains this design concept nicely. 
Though the tutorial is for a previous Cocos2D project specifically, in the middle of it (about) there are 3 explanations of ways to implement this design.
The first requires many on/off switches for components, which is not library-friendly.
The second makes cross-component interaction difficult, which is not optimal.
The third and final implementation is perfect for a library, due to its expandable and flexible approach.
This is the design that Ruby-CES uses.

In a CES system, all 'objects' in the game/app are **entities**, each with a unique identifier (Entity IDentifier).
These entities do not keep track of their own components.
**Components** are simply holders of data, for example (seen in Usage below) a `meta` section, holding a `name` property, or some random other key:value pair, like `append`.
**Systems** process a given component, defined using a block.
When being run, systems are passed 2 arguments: the component being processed (to access data) and the EID of the current entity which this component belongs to.

A class called ComponentReference holds arrays for each component, and each array holds which EIDs have that component.
With that in place, another class, SystemReference, holds the system definitions (blocks and names).
When `SystemReference.tick` is called, it takes a system, then gives that system - one at a time - the component and EID from the list of EIDs that have the current system's component, and processes them all, and finally repeats that with all the systems.
At the end of all your CES programs you should call `SystemReference.run(n)`, which loops the tick method, ticking the game/app every `n` seconds.

Please open an issue if there is an issue that needs to be fixed, or if you need help with a certain thing, and I will try my best to help you.
Additionally, near the end of this README there is a section detailing the development of this project and how to contribute to it, if you would like to.

## Installation

Add this line to your application's Gemfile:

```ruby
gem 'ces'
```

And then execute:

```
$ bundle
```
	
Or install it yourself with:

```
$ gem install ces
```
	
## Usage

Very simple component system, demonstrating how to use CES in general and also how to use data stored in the component.
The following code bit will - every 2 seconds - print the EID (Entity IDentifier) of the entity which the component belongs to, current time, and a custom message to the console.

```ruby
### In ./components/print_current_time_with_append.json` - a JSON file holding AT LEAST a meta section, containing AT LEAST a name; this file can also have it's own accessible key:value pairs outside of the meta tag
{
	"meta": {
		"name":"print_current_time_with_append"
	},
	"append":". Have a nice day :)"
}

### In ./systems/print_current_time_with_append.rb
# Register a system (processor) for "print_current_time_with_append"
CES::SystemReference.register("print_current_time_with_append") { |component, eid| # All systems are passed 2 arguments, the component being processed and the Entity ID being processed
	puts "[EID#{eid}] #{Time.now.utc.asctime}#{component["append"]}" # The - simple - logic of our example 
}

### In example_ces_program.rb
# Load the library - necessary for all scripts using this library.
require 'ces'

# Load both the components (from ./components) and the systems (from ./systems)
CES.loadcomponents
CES.loadsystems

# Create a new Entity with a single component, "print_current_time_with_append"
CES::Entity.new("print_current_time_with_append")

# Run the loop, updating all components every 2 seconds
CES::SystemReference.run(2)
```

## Development/Contributing

Please read the [code of conduct](CODE_OF_CONDUCT.md) before submitting any PRs.
After checking out the repo, run `gem install json` to install the only current dependency.

Bug reports and pull requests are welcome on GitHub at https://github.com/Punknoodles/ruby-ces.
This project is intended to be a safe, welcoming space for collaboration, and contributors are expected to adhere to the [Contributor Covenant](http://contributor-covenant.org) code of conduct, found [here](CODE_OF_CONDUCT.md) .

Everyone interacting in the CES project’s codebases, issue trackers, chat rooms and mailing lists is expected to follow the [code of conduct](https://github.com/Punknoodles/ces/blob/master/CODE_OF_CONDUCT.md).

Here is a small list of things that are (hopefully) going to get done sometime, that _you_ might be able to do!

* Cache the current entity data, and use that data when running the systems, so that data doesn't get changed mid-tick and a different outcome occur because of it
* Comment and clean up the code base
* Make more examples for other use cases

## License

The gem is available as open source under the terms of the [MIT License](http://opensource.org/licenses/MIT).
